=begin

curve_tools.rb: (C) 2009 Amos Bieler
("find_center" method based on code by Chris Fullmer, 
because my version was ugly ;)

-=_+	Description:

This Ruby code can be useful to both modelers and plugin writers.

-=_+	Modelers:

As a tool for modelers, it is used to draw a construction 
point at either the center point of a Curve, ArcCurve or 
Circle entity, or at the point where the perpendiculars of 
two Edges intersect. The latter effect is handy when an 
ArcCurve or Circle has been exploded for any reason, such as 
being Push/Pull'ed.

Usage: Select and context-click on a curve or at least two 
Edges and select The option ending in the phrase 
"Center-point" (it changes depending on what is selected).

-=_+	Authors:

As a tool for plugin authors, it adds the "center" and 
"radius" methods to Curve objects. In SketchUp's standard 
API, these methods are only available to the ArcCurve class 
and its derivatives. It also adds a "find_center" method to 
the Edge class to facilitate the above-mentioned effects.

Usage:

	require 'curve_tools'

	puts curve_object.center.to_s
	(0.0", 0.0", 0.0")
	nil

	-or-

	puts edge_object.find_center(other_edge).to_s
	(0.0", 0.0", 0.0")
	nil

-=_+	History

2009-08-10 - Initial release

=end

require 'sketchup.rb'

# Add some needed functionality to the "Array" class
class Array

# Returns the average of all Numeric objects in the array
def average
	rt = 0.to_f
	c = 0
	self.each { |e|
		if e.is_a?( Numeric ) then
			rt = (rt + e.to_f)
			c += 1
		end
	}
	rt / c
end

end

class Sketchup::Edge

def find_center (seg)
#		puts "find_center(#{seg})"
		return nil if (seg.nil?)
		return nil if (seg.typename != 'Edge')
		s1p1 = self.start.position
		s1p2 = self.end.position
		s2p1 = seg.start.position
		s2p2 = seg.end.position
		orig_vec1 = self.line[1]
		orig_vec2 = seg.line[1]
		return nil if (orig_vec1.parallel? orig_vec2)
		mp1 = Geom.linear_combination 0.5, s1p1, 0.5, s1p2
		mp2 = Geom.linear_combination 0.5, s2p1, 0.5, s2p2
		zvec = orig_vec1.cross orig_vec2
		vec1 = orig_vec1.cross zvec
		vec2 = orig_vec2.cross zvec

		line1 = [mp1, vec1]
		line2 = [mp2, vec2]
		pt = Geom.intersect_line_line(line1, line2)
		if pt.nil?
			vec1.reverse!
			vec2.reverse!
			line1 = [mp1, vec1]
			line2 = [mp2, vec2]
			pt = Geom.intersect_line_line(line1, line2)
		end
		pt
end # find_center

end # Edge class

class Sketchup::Curve

def center
	e1 = self.edges.clone
	e2 = e1.clone
	e2.push e2.shift
	e1.each_index { |i|
#		puts "#{e1[i].length} & #{e2[i].length}"
		return( e1[i].find_center( e2[i] ) ) if (e1[i].length == e2[i].length)
	}
	nil
end # center

def radius
	e1 = self.edges.clone
	e2 = e1.clone
	e2.push e2.shift
	e1.each_index { |i|
#		puts "#{e1[i].length} & #{e2[i].length}"
		return( self.center.distance( e1[i].end ) ) if (e1[i].length == e2[i].length)
	}
	nil
end # radius

def minor_radius
	radii = []
	self.vertices.each { |v|
		radii.push( self.center.distance(v) )
	}
	radii.min
end # minor_radius

def major_radius
	radii = []
	self.vertices.each { |v|
		radii.push( self.center.distance(v) )
	}
	radii.max
end # major_radius

end # Curve class

unless file_loaded? File::basename(__FILE__)

	UI.add_context_menu_handler do |menu|
		sel = Sketchup.active_model.selection
		if (sel[0].respond_to?('curve'))
			curve = sel[0].curve
			if (curve.respond_to?('center'))
				menu.add_separator
				menu.add_item('Draw Center-point of Curve, ArcCurve, or Circle') {
					cpt = curve.center
					puts "it's nil" if cpt.nil?
					Sketchup.active_model.active_entities.add_cpoint(cpt) unless cpt.nil?
				}
			end
		end
		if (sel.length > 1 and
			sel[0].typename == 'Edge' and
			sel[-1].typename == 'Edge' and
			sel[0].curve.nil? and
			sel[-1].curve.nil?
		)
			menu.add_separator
			menu.add_item("Draw Focal-point of selected Edges") {
				cpt = sel[0].find_center( sel[-1] )
				Sketchup.active_model.active_entities.add_cpoint(cpt) unless cpt.nil?
			}
		end
	end

	file_loaded File::basename(__FILE__)

end
